#!/bin/bash

#variables
TARGET=/tmp/cxacru-deb-pkg
VERSION=`date +%Y-%m-%d`_`uname -r`
MAINTAINER="Josep Comas <jcomas@gna.es>"
DATE=`date -R`
if [ "$1" = "" ]; then
  DEST_MACHINE=`uname -m`
else
  DEST_MACHINE=$1
fi
FILE_CONTROL="$TARGET"/DEBIAN/control
FILE_COPYRIGHT="$TARGET"/DEBIAN/copyright
FILE_CHANGELOG="$TARGET"/DEBIAN/changelog
FILE_CONFFILES="$TARGET"/DEBIAN/conffiles
FILE_POSTINST="$TARGET"/DEBIAN/postinst
FILE_PRERM="$TARGET"/DEBIAN/prerm
SOURCE_BIN=/usr/sbin
SOURCE_MODULE=/lib/modules/`uname -r`/kernel/drivers/usb
PACKAGE_NAME=../../cxacru-`date +%Y-%m-%d`.k`uname -r`."$DEST_MACHINE".deb
SHLIBDEPS=`dpkg-shlibdeps -O $SOURCE_BIN/cxload $SOURCE_BIN/cxioctl $SOURCE_BIN/br2684ctl $SOURCE_BIN/cxpanel | sed 's/shlibs:Depends=//g'` 


#
#create DEBIAN directory and create files to build deb package
#

rm -rf $TARGET
mkdir -p $TARGET/DEBIAN

#Architecture: $DEST_MACHINE changed by Architecture: i386

cat > $FILE_CONTROL << EOF
Source: cxacru
Section: contrib/net
Priority: optional
Maintainer: $MAINTAINER
Package: cxacru
Architecture: i386
Version: $VERSION
Depends: $SHLIBDEPS
Description: Conexant AccessRunner USB ADSL modem kernel space driver
 This package contains software to use Conexant AccessRunner USB ADSL modem
 under Linux. It supports several protocols.
EOF

cat > $FILE_COPYRIGHT << EOF
$DATE

It was downloaded from: http://sourceforge.net/projects/accessrunner/

Upstream author: $MAINTAINER

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License as
published by the Free Software Foundation; either version 2 of
the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

On Debian systems, the complete text of the GNU General Public
License can be found in /usr/share/common-licenses/GPL file.
EOF

cat > $FILE_CHANGELOG << EOF
cxacru ($VERSION) unstable; urgency=low

  * Maintaining the Debian package myself.

 -- $MAINTAINER  $DATE 
EOF

cat > $FILE_CONFFILES << EOF
/etc/cxacru
EOF

cat > $FILE_POSTINST << EOF
#!/bin/bash
/sbin/depmod -a
if [ -f /etc/debian_version ]; then 
  install -c -m 755 -p /etc/cxacru.service /etc/init.d/cxacru; 
  update-rc.d cxacru defaults; 
  update-rc.d -f atm remove; 
else 
  if [ -f /etc/redhat-release ]; then 
    install -c -m 755 -p /etc/cxacru.service /etc/rc.d/init.d/cxacru; 
    chkconfig --add cxacru; 
  else 
    if [ -f /etc/SuSE-release ] ; then 
      install -c -m 755 -p /etc/cxacru.service /etc/init.d/cxacru; 
      install -c -m 755 -p /etc/cxacru.service /etc/rc.d/cxacru; 
      insserv cxacru; 
    else 
      if [ -f /etc/slackware-version ] ; then 
        if grep -q -E "^ */usr/sbin/cxstart.sh *$" /etc/rc.d/rc.local; then 
          cd /etc/rc.d; 
          cp -f rc.local rc.local.tmp; 
          echo "/usr/sbin/cxstart.sh" >> rc.local.tmp; 
          mv -f rc.local.tmp rc.local; 
        fi 
      fi 
    fi 
  fi 
fi
EOF

cat > $FILE_PRERM << EOF
#!/bin/bash
if [ -f /etc/debian_version ]; then 
        update-rc.d -f cxacru remove; 
        rm -f /etc/init.d/cxacru; 
else 
        if [ -f /etc/redhat-release ]; then 
                chkconfig --del cxacru; 
                rm -f /etc/rc.d/init.d/cxacru; 
        else 
                if [ -f /etc/SuSE-release ] ; then 
                        insserv -r cxacru; 
                        rm -f /etc/init.d/cxacru; 
                        rm -f /etc/rc.d/cxacru; 
                else 
                        if [ -f /etc/slackware-version ] ; then 
                                cd /etc/rc.d; 
                                grep -v -E "^ */usr/sbin/cxstart.sh *$" rc.local > rc.local.tmp; 
                                mv -f rc.local.tmp rc.local; 
                        fi 
                fi 
        fi 
fi
EOF

chmod 644 $FILE_CONTROL
chmod 644 $FILE_COPYRIGHT
chmod 644 $FILE_CHANGELOG
chmod 644 $FILE_CONFFILES
chmod 755 $FILE_POSTINST
chmod 755 $FILE_PRERM


#
#copy driver files in temp directory for deb package
#

mkdir -p $TARGET$SOURCE_BIN
mkdir -p $TARGET$SOURCE_MODULE
mkdir -p $TARGET/etc

#init
cp -p $SOURCE_BIN/cxload $TARGET$SOURCE_BIN 
cp -p $SOURCE_BIN/cxioctl $TARGET$SOURCE_BIN

#firmware
cp -p $SOURCE_BIN/cxfirm1.bin $TARGET$SOURCE_BIN
cp -p $SOURCE_BIN/cxfirm2.bin $TARGET$SOURCE_BIN
cp -p $SOURCE_BIN/cxfirm3.bin $TARGET$SOURCE_BIN
cp -p $SOURCE_BIN/cxfirm4.bin $TARGET$SOURCE_BIN
cp -p $SOURCE_BIN/cxfirm5.bin $TARGET$SOURCE_BIN

#module
KERNEL_VERSION=`uname -r | cut -d'.' -f1-2`
if [ "$KERNEL_VERSION" = "2.4" ]; then
  cp -p $SOURCE_MODULE/cxacru.o $TARGET$SOURCE_MODULE
  cp -p $SOURCE_MODULE/cxacrudbg.o $TARGET$SOURCE_MODULE
else
  cp -p $SOURCE_MODULE/cxacru.ko $TARGET$SOURCE_MODULE
  cp -p $SOURCE_MODULE/cxacrudbg.ko $TARGET$SOURCE_MODULE
fi

#scripts
cp -p $SOURCE_BIN/cxstart.sh $TARGET$SOURCE_BIN
cp -p $SOURCE_BIN/cxstop.sh $TARGET$SOURCE_BIN
cp -p $SOURCE_BIN/cxload.sh $TARGET$SOURCE_BIN
cp -p $SOURCE_BIN/cxunload.sh $TARGET$SOURCE_BIN
cp -p $SOURCE_BIN/cxnetup.sh $TARGET$SOURCE_BIN
cp -p $SOURCE_BIN/cxnet2up.sh $TARGET$SOURCE_BIN
cp -p $SOURCE_BIN/cxnet3up.sh $TARGET$SOURCE_BIN
cp -p $SOURCE_BIN/cxnet4up.sh $TARGET$SOURCE_BIN
cp -p $SOURCE_BIN/cxnetdown.sh $TARGET$SOURCE_BIN
cp -p $SOURCE_BIN/cxnet2down.sh $TARGET$SOURCE_BIN
cp -p $SOURCE_BIN/cxnet3down.sh $TARGET$SOURCE_BIN
cp -p $SOURCE_BIN/cxnet4down.sh $TARGET$SOURCE_BIN

#config
cp -p /etc/cxacru $TARGET/etc

#service
cp -p /etc/cxacru.service $TARGET/etc

#bridged
cp -p $SOURCE_BIN/br2684ctl $TARGET$SOURCE_BIN

#panel
cp -p $SOURCE_BIN/cxpanel $TARGET$SOURCE_BIN
cp -p $SOURCE_BIN/cxpanel.glade $TARGET$SOURCE_BIN
cp -p $SOURCE_BIN/cxacru-tux.xpm $TARGET$SOURCE_BIN


#
#build deb package
#

dpkg -b $TARGET $PACKAGE_NAME 
rm -rf $TARGET

